use zerocopy::{AsBytes, FromBytes, FromZeroes, Unaligned};

use super::boolean::Boolean;

#[derive(Clone, Copy, FromBytes, FromZeroes, AsBytes, Unaligned)]
#[repr(packed)]
pub struct Optional<T> {
    present: Boolean,
    value: T,
}

impl<T: FromZeroes> From<Option<T>> for Optional<T> {
    fn from(value: Option<T>) -> Self {
        match value {
            Some(value) => Optional::some(value),
            None => Optional::none(),
        }
    }
}

impl<T: FromZeroes> Default for Optional<T> {
    fn default() -> Self {
        Self::none()
    }
}

impl<T> Optional<T> {
    pub fn none() -> Self
    where
        T: FromZeroes,
    {
        Self {
            present: Boolean::new(false),
            value: T::new_zeroed(),
        }
    }
    pub fn some(value: T) -> Self {
        Self {
            present: Boolean::new(true),
            value,
        }
    }
    pub fn get(self) -> Option<T> {
        if self.present.get() {
            Some(self.value)
        } else {
            None
        }
    }

    pub fn is_some(&self) -> bool {
        self.present.get()
    }

    pub fn is_none(&self) -> bool {
        !self.is_some()
    }
}
